!
subroutine DE21(DS, X, Xout, H, Eps, Y, Error)
!     DE28R  DE29R  :
! ...  ..   
!   . . . , 1990.
interface
  subroutine DS(X, Y, DY)
  real, intent(in):: X, Y(:)
  real:: DY(:)
  end subroutine DS
end interface
real, intent(in):: Xout
real, intent(inout):: X, H, Eps, Y(:)
integer:: Error
integer, parameter:: Order=4
real, parameter:: C1=0.5, C2=5.0/12.0, C3=0.375
real, parameter:: P=-19.0/720.0
real, parameter:: ARB=0.001, OneP=1.000001, EEE=0.0625
real, allocatable:: Y0(:), YH(:), DY(:), F(:), DF(:,:), W(:)
real:: HM4, X0, XH, DX, Hmin, SaveH, R, E, E32, Err, D
integer:: i, j, k, M, N, Count
logical:: HChange
!begin
  if (X==Xout) then
    H=0.0; Error=1; return
  end if
  M=size(Y)
  allocate(F(M))
  HM4=4.0*epsilon(1.0)
  if (abs(X)>ARB) then; Hmin=HM4*abs(X)
                  else; Hmin=HM4*ARB
  end if
  DX=Xout-X
  if (abs(DX)<float(Order)*Hmin) then
    H=DX
    call DS(X, Y, F)
    Y(:)=Y(:)+DX*F(:)
    deallocate(F)
    X=Xout; Error=1; return
  end if
  if (Eps<EEE*epsilon(1.0)) then
    Eps=EEE*epsilon(1.0)
    deallocate(F)
    Error=2; return
  end if
  if (abs(H)<Hmin) H=Hmin
  if (float(Order)*abs(H)>abs(DX)) then
    H=DX/float(Order)
  else; H=sign(H, DX)
  end if
  allocate(DF(1:M,0:Order), DY(M), Y0(M), YH(M), W(M))
  call DE19(DS, X, Y, H, M, Order, DF, XH, YH, Err)
  X0=X; Y0=Y
  !        0
  call AdamsOneStep(DS, X0, Y0, H, M, Order, DF, Y, Err)
  if(Err>Eps) then
    ! ,      Eps
    Y=Y0; deallocate(F, DF, DY, Y0, YH, W)
    Error=3; return
  end if
  ! ,   
  X=X0+H
  E=Eps/(-P); E32=E*0.03125
  N=0; Count=0; Error=0
  HChange=.false.
10 continue
  X0=X; Y0(:)=Y(:)
20 continue
  X=X0+H
  Y(:)=Y0(:)+DF(:,0)+C1*DF(:,1)+C2*DF(:,2)+C3*DF(:,3)
  call DS(X, Y, F)
  F(:)=H*F(:)
  DY(:)=DF(:,0)+DF(:,1)+DF(:,2)+DF(:,3)-F(:)
  Y(:)=Y(:)-C3*DY(:)
  DY(:)=P*DY(:)
  do i=1, M
    if (abs(Y(i))>ARB) then; D=abs(Y(i))
                        else; D=ARB
    end if
    D=abs(DY(i))/D
    if (D>E) goto 30
  end do
  goto 40
30 continue
  if (abs(X0)>ARB) then; Hmin=HM4*abs(X0)
                   else; Hmin=HM4*ARB
  end if
  if (abs(H)<Hmin*OneP) then
    X=X0; Y=Y0
    deallocate(Y0, YH, DY, F, DF, W)
    Error=65; return
  end if
  N=0; Count=0
  R=0.5; H=0.5*H
  if (abs(H)<Hmin) then
    H=sign(Hmin, H); R=H/(X-X0)
  end if
  call DE21P(M, Order, R, DF, DF)
  goto 20
40 continue
  do j=0, Order-1
    W(:)=DF(:,j); DF(:,j)=F(:); F(:)=F(:)-W(:)
  end do
  DF(:,Order)=F(:)
  Y(:)=Y(:)+P*F(:)
  if (N>0) then
    N=N-1
    if (N==0) then
      deallocate (Y0, YH, DY, F, DF, W)
      H=SaveH; return
    end if
    goto 10
  end if
  do i=1, M
    if (abs(DY(i)/Y(i))>E32) then
      Count=0; goto 50
    end if
  end do
  Count=Count+1
  if (Count==Order) then
    Count=0
    R=2.0; H=2.0*H
    HChange=.true.
  end if
50 continue
  DX=Xout-X
  if (2.0*abs(H)>abs(DX)) then
    SaveH=H; HChange=.true.
    if (abs(H)>=abs(DX)) then
      R=DX/H; H=DX; N=1
    else
      R=(0.5*DX)/H; H=0.5*DX; N=2
    end if
  end if
  if (HChange) then
    HChange=.false.
    call DE21P(M, Order, R, DF, DF)
  end if
  goto 10
!end DE21

contains

subroutine AdamsOneStep(DS, X0, Y0, H, M, Order, DF, Y, Err)
!*      Order 
!*   DS   0     
interface
  subroutine DS(X, Y, DY)
  real, intent(in):: X, Y(:)
  real:: DY(:)
  end subroutine DS
end interface
integer, intent(in):: M, Order
real, intent(in):: X0, Y0(M), H
real, intent(inout):: DF(1:M,0:Order)
real:: Y(M), Err
real, allocatable:: DY(:), F(:), W(:)
integer, parameter:: ARB=0.001, MaxOrder=6
real, parameter:: C(0:MaxOrder-1)=(/1.0, 0.5, 5.0/12.0, 0.375,    &
  251.0/720.0, 95.0/288.0/)
real, parameter:: P(MaxOrder)=(/-0.5, -1.0/12.0, -1.0/24.0,  &
  -19.0/720.0, -3.0/160.0, -863.0/60480.0/)
integer:: j
real:: X
!begin
  allocate(DY(M), F(M), W(M))
  X=X0+H
  Y(:)=Y0(:)+DF(:,0); DY(:)=DF(:,0)
  do j=1, Order-1
    Y(:)=Y(:)+C(j)*DF(:,j)
    DY(:)=DY(:)+DF(:,j)
  end do
  call DS(X, Y, F)
  F(:)=H*F(:)
  Y(:)=Y(:)-C(Order-1)*DY(:)+C(Order-1)*F(:)
  do j=0, Order-1
    W(:)=DF(:,j); DF(:,j)=F(:); F(:)=F(:)-W(:)
  end do
  DF(:,Order)=F(:)
  Y(:)=Y(:)+P(Order)*F(:)
  Err=0.0
  do i=1, M
    if (abs(Y(i))>ARB) then; R=abs(Y(i))
                        else; R=ARB
    end if
    R=abs(F(i))/R
    if (Err<R) Err=R
  end do
  Err=-P(Order)*Err
  deallocate(DY, F, W)
  return
end subroutine AdamsOneStep

subroutine DE21P(M, Order, R, DFin, DFout)
integer, intent(in):: M, Order
real, intent(in):: R, DFin(1:M,0:Order)
real:: DFout(1:M,0:Order)
real, allocatable:: S(:)
real:: Rm1, C(21)
integer:: i, j, k, k1
!begin
  if (R==1.0) return
  if (R==0.5) then
    C(1)=0.25
    if (Order>=2) then
      C(2)=0.0625; C(3)=0.125
      if (Order>=3) then
        C(4)=0.03125; C(5)=0.0625; C(6)=0.0625
          if (Order>=4) then
            C(7)=5.0/256.0; C(8)=5.0/128.0; C(9)=3.0/64.0; C(10)=0.03125
              if (Order>=5) then
                C(11)=7.0/512.0; C(12)=7.0/256.0; C(13)=9.0/256.0
                C(14)=0.03125; C(15)=0.015625
              else
                C(16)=21.0/2048.0; C(17)=21.0/1024.0
                C(18)=7.0/256.0; C(19)=7.0/256.0
                C(20)=5.0/256.0; C(21)=0.0078125
    endif; endif; endif; endif
  else if (R==2.0) then
    C(1)=4.0
    if (Order>=2) then
      C(2)=-2.0; C(3)=8.0
      if (Order>=3) then
        C(4)=0.0; C(5)=-8.0; C(6)=16.0
          if (Order>=4) then
            C(7)=0.0; C(8)=2.0; C(9)=-24.0; C(10)=32.0
              if (Order>=5) then
                C(11)=0.0; C(12)=0.0; C(13)=12.0; C(14)=-64.0; C(15)=64.0
              else
                C(16)=0.0; C(17)=0.0; C(18)=-2.0; C(19)=48.0
                C(20)=-160.0; C(21)=128.0
    endif; endif; endif; endif
  else
    C(1)=R*R
    if (Order>=2) then
      Rm1=R-1.0
      C(2)=-0.5*C(1)*Rm1; C(3)=C(1)*R
      if (Order>=3) then
        C(4)=-(1.0/3.0)*C(2)*(R-2.0); C(5)=-C(3)*Rm1; C(6)=C(3)*R
          if (Order>=4) then
            C(7)=-0.25*C(4)*(R-3.0); C(8)=-(7.0/12.0)*C(5)*(R-(11.0/7.0))
            C(9)=-1.5*C(6)*Rm1; C(10)=C(6)*R
              if (Order>=5) then
                C(11)=-0.2*C(7)*(R-4.0); C(12)=-1.5*C(4)*R*(R-(5.0/3.0))
                C(13)=-(5.0/6.0)*C(9)*(R-1.4); C(14)=-2.0*C(10)*Rm1
                C(15)=C(10)*R
              else
                C(16)=-(1.0/6.0)*C(11)*(R-5.0)
                C(17)=(31.0/60.0)*C(4)*R*((R-(132.0/31.0))*R+(137.0/31.0))
                C(18)=0.5*C(9)*((R-(19.0/6.0))*R+2.5)
                C(19)=-(13.0/12.0)*C(14)*(R-(17.0/13.0))
                C(20)=-2.5*C(15)*Rm1; C(21)=C(15)*R
    endif; endif; endif; endif
  endif
  allocate (S(M))
  DFout(:,0)=DFin(:,0)*R
  do j=1, Order
    S(:)=0.0
    do k=j, Order
      k1=k*(k-1)/2+j
      S(:)=S(:)+C(k1)*DFin(:,k)
    end do
    DFout(:,j)=S(:)
  end do
  deallocate(S)
  return
end subroutine DE21P

end subroutine DE21
